# -*-coding:utf-8 -*-
u"""
:创建时间: 2021/12/25 13:30
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function
import json
import os.path
import sys

from pyeal.utils import read_string, write_string
from pyeal.res import LocalRes, DirectoryRes
from pyeal.core import EncapsulationBuilder, InstallBuilder

PATH = os.path.dirname(os.path.abspath(__file__))


class SealException(Exception):
    pass


MayaPlugin = "MayaPlugin"


class Config(object):
    def __init__(self, root):
        root = os.path.abspath(root)
        self.root = root
        root_res = LocalRes(root)
        config = json.loads(root_res.read("config.json").decode("utf-8"))
        self.type = config["type"]
        self.source = root_res.sep().join((root, config["source"]))
        self.target = root_res.sep().join((root, config["target"]))
        self.name = config["name"]
        self.exec_script = config["exec_script"]
        self.log = config["log"]
        self.annotation = config["annotation"]
        self.ord = config

    def __str__(self):
        return """{}<type={}, source={}, target={}>""".format(self.__class__.__name__, self.type, self.source,
                                                              self.target)


def go(root):
    root = os.path.abspath(root)
    config = Config(root)
    if config.type == MayaPlugin:
        root = LocalRes(config.root)
        src = LocalRes(config.source)
        build = LocalRes(config.target)
        m0 = DirectoryRes(build, "m0")
        dist = DirectoryRes(build, "dist")
        EncapsulationBuilder(src, m0, config.name, config.exec_script).build()
        InstallBuilder(m0, dist, log=root.read(config.log), ann=config.annotation).build()
    else:
        raise SealException("未知编译类型")


config_json_template = r'''{
  "type": "MayaPlugin",
  "source": "src",
  "target": "build",
  "name": <<name>>,
  "exec_script": <<exec_script>>,
  "log": "log.ico",
  "annotation": "这是这个插件的注释~"
}'''


def input_string(s):
    sys.stdout.write(s)
    return sys.stdin.readline().strip("\n")


def init(root):
    root = os.path.abspath(root)
    name = input_string("name >> ")
    exec_script = input_string("exec_script >> ")
    name = json.dumps(name)
    exec_script = json.dumps(exec_script)
    config_json = config_json_template.replace("<<name>>", name).replace("<<exec_script>>", exec_script)
    config_json_path = os.sep.join((root, "config.json"))
    with open(config_json_path, "wb") as f:
        f.write(config_json.encode("utf-8"))

    with open(os.sep.join((PATH, "log.ico")), "rb") as f:
        with open(os.sep.join((root, "log.ico")), "wb") as wf:
            wf.write(f.read())
    src = os.sep.join((root, "src"))
    if not os.path.isdir(src):
        os.makedirs(src)
    build = os.sep.join((root, "build"))
    if not os.path.isdir(build):
        os.makedirs(build)


def main():
    argv = sys.argv[1:]
    root = os.path.abspath(".")
    if argv[0] == "init":
        init(root)
    elif argv[0] == "go":
        go(root)
    else:
        raise SealException("未知指令")


if __name__ == "__main__":
    main()



# -*-coding:utf-8 -*-
u"""
:创建时间: 2021/12/24 15:12
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function

from pyeal.res import BaseRes


def file_ext(filename, sep='/'):
    return "." + ".".join(filename.split(sep)[-1].split(".")[1:])


def file_name(filename):
    return filename.split(".")[0]


class ModuleDataException(Exception):
    pass


class ModuleData(object):
    def __init__(self, source):
        """
        :type source: BaseRes
        """
        self.source = source
        self._check()
        self._module_index_table = self._init_module_index_table()
        self._lib_names = self._init_lib_names()
        pass

    def _check(self):
        module_names = set()
        for root, files in self.source.walk():
            if not root is None and "__init__.py" in files:
                module_name = ".".join(root.split(self.source.sep()))
                if module_name in module_names:
                    raise ModuleDataException("存在多个解析名称为{}的文件或文件夹".format(module_name))
                module_names.add(module_name)
            if root is None:
                for f in files:
                    if file_ext(f, self.source.sep()) == ".py":
                        module_name = file_name(f)
                        if module_name in module_names:
                            raise ModuleDataException("存在多个解析名称为{}的文件或文件夹".format(module_name))
                        module_names.add(module_name)

    def _init_module_index_table(self):
        table = dict()
        for root, files in self.source.walk():
            # 检查包
            if root is not None and "__init__.py" in files:
                table[".".join(root.split(self.source.sep()))] = self.source.sep().join((root, "__init__.py"))
            # 检查模块
            for f in files:
                if file_ext(f, self.source.sep()) == ".py":
                    f_name = file_name(f)
                    if root is None:
                        table[f_name] = f
                    else:
                        f_name = ".".join(self.source.sep().join((root, f_name)).split(self.source.sep()))
                        table[f_name] = self.source.sep().join((root, f))

        return table

    def _init_lib_names(self):
        return set((i.split(".")[0] for i in self._module_index_table.keys()))

    def lib_names(self):
        return (i for i in self._lib_names)

    def module_names(self):
        return (i for i in self._module_index_table.keys())

    def module_name_and_paths(self):
        return ((k, v) for k, v in self._module_index_table.items())

    def find_lib(self, n):
        if n in self._lib_names:
            return n
        return None

    def find_module(self, n):
        if n in self._module_index_table:
            return n
        return None

    def relative_find_module(self, n, parent_module):
        parent_names = parent_module.split(".")
        for m in self._module_index_table.keys():
            if len(m.split(".")) > len(parent_names):
                if parent_module == ".".join(m.split(".")[:len(parent_names)]):
                    if n == ".".join(m.split(".")[len(parent_names):]):
                        return m
        return None

    def get_module_path(self, n):
        return self._module_index_table[n]

    def __str__(self):
        body = "\n    ".join(("{}: {}".format(k, v) for k, v in self._module_index_table.items()))
        return self.__class__.__name__ + "{\n    %s\n}" % body

    def __repr__(self):
        return repr(self.__str__())



# -*-coding:utf-8 -*-
u"""
:创建时间: 2021/12/19 9:54
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function

import os


def read(filename):
    """
    :type filename: unicode
    """
    with open(filename, 'rb') as f:
        return f.read()


def write(filename, b):
    """

    :type filename: unicode
    :type b: bytes
    """
    with open(filename, 'rb') as f:
        return f.write(b)


def read_string(filename):
    """
    :type filename: unicode
    """
    return read(filename).decode("utf-8")


def write_string(filename, s):
    """

    :type filename: unicode
    :type s: unicode
    """
    write(filename, s.encode("utf-8"))


def all_files(path):
    return (os.sep.join((root, f)) for root, dirs, fs in os.walk(path) for f in fs)


def all_partial_paths(path):
    path_size = len(path.split(os.sep))
    return (os.sep.join(f.split(os.sep)[path_size:]) for f in all_files(path))


def all_full_and_partial_paths(path):
    path_size = len(path.split(os.sep))
    return ((f, os.sep.join(f.split(os.sep)[path_size:])) for f in all_files(path))


def all_lib_names(path):
    """获得所有库的名称（不是模块或者包的，并不会深入包内部）"""
    for f, p in all_full_and_partial_paths(path):
        items = p.split(os.sep)
        if len(items) == 1:
            i = items[0]
            split_char_index = i.find(".")
            ext = i[split_char_index:]
            if ext == ".py" or ".pyd" or ".pyo":
                yield i[:split_char_index], f
        elif len(items) == 2:
            if items[1] == "__init__.py":
                yield items[0], f.split(os.sep)[:-1]


__all__ = ['read_string', 'write_string', 'all_files', 'all_partial_paths', 'all_full_and_partial_paths',
           'all_lib_names']
